<?php

namespace Drupal\rmkv\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Extension\ThemeHandler;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\KeyValueStore\KeyValueFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implementing system configuration forms.
 */
class RmKVForm extends ConfigFormBase {

  /**
   * Key/value storage.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueFactory
   */
  protected $keyValue;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ThemeHandler
   */
  protected $themeHandler;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\KeyValueStore\KeyValueFactory $key_value
   *   The key/value storage collection.
   * @param \Drupal\Core\Extension\ModuleHandler $module_handler
   *   The module handler.
   * @param \Drupal\Core\Extension\ThemeHandler $theme_handler
   *   The theme handler.
   */
  public function __construct(ConfigFactoryInterface $config_factory, KeyValueFactory $key_value, ModuleHandler $module_handler, ThemeHandler $theme_handler) {
    $this->setConfigFactory($config_factory);
    $this->keyValue = $key_value;
    $this->moduleHandler = $module_handler;
    $this->themeHandler = $theme_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('keyvalue'),
      $container->get('module_handler'),
      $container->get('theme_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'custom_rmkv_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Form constructor.
    $form = parent::buildForm($form, $form_state);

    // Source text field.
    $form['system_schema_key_value_machine_name'] = [
      '#type' => 'machine_name',
      '#title' => $this->t('Machine name of the system.schema key/value storage for remove:'),
      '#machine_name' => [
        'exists' => [$this, 'validateMachineName'],
        'source' => [],
      ],
    ];

    // Submit button value.
    $form['actions']['submit']['#value'] = $this->t('Remove system.schema key/value storage');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateMachineName($machine_name, $element, FormStateInterface $form_state) {
    // Get system schema.
    $system_schema = $this->keyValue->get('system.schema');

    // Check of machine name.
    if (!$system_schema->has($machine_name)) {
      $message = $this->t('Specified machine name "@machine_name" is not exists to the system.schema key/value storage.', [
        '@machine_name' => $machine_name,
      ]);
      $form_state->setError($element, $message);
    }
    else {
      if ($this->moduleHandler->moduleExists($machine_name) || $this->themeHandler->themeExists($machine_name)) {
        $message = $this->t('Cannot specify the machine name "@machine_name" of the installed module or theme. Specify the machine name of the uninstalled module or theme.', [
          '@machine_name' => $machine_name,
        ]);
        $form_state->setError($element, $message);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Get system schema.
    $system_schema = $this->keyValue->get('system.schema');

    // Get machine name of input value.
    $machine_name = $form_state->getValue('system_schema_key_value_machine_name');

    // Check of machine name.
    if ($system_schema->has($machine_name)) {
      if (!$this->moduleHandler->moduleExists($machine_name) && !$this->themeHandler->themeExists($machine_name)) {
        $system_schema->delete($machine_name);
        $this->messenger()->addStatus($this->t('Succeeded in removing "@machine_name" from system.schema key/value storage.', [
          '@machine_name' => $machine_name,
        ]));
      }
      else {
        $this->messenger()->addError($this->t('Aborted remove of "@machine_name" from system.schema key/value storage, because specified machine name is already installed.', [
          '@machine_name' => $machine_name,
        ]));
      }
    }
    else {
      $this->messenger()->addWarning($this->t('Specified machine name "@machine_name" is not exists to the system.schema key/value storage. (* This message is displayed if it may have already been deleted.)', [
        '@machine_name' => $machine_name,
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'rmkv.form',
    ];
  }

}
